#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <unistd.h>

#include "util.h"
#include "cable.h"
#include "serjtag.h"
#include "ft245r.h"
#include "jtag.h"

#include "target.h"
#include "tgtlscc.h"
#include "tgtxil.h"

char *program;
char *progname;
int verbose = 1;

CABLE cbl;
TARGET tgt;

static void jtag_test(CABLE cbl)
{
	uint32_t r;
	double st,et;

	st = dtime();
	usleep(200000);
	et = dtime();
	printf("usleep 200ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

    if (0 && cbl->delay) {
	st = dtime();
	(cbl->delay)(cbl, 4000);
	et = dtime();
	printf("delay 4ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	(cbl->delay)(cbl, 9000);
	et = dtime();
	printf("delay 9ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	(cbl->delay)(cbl, 20000);
	et = dtime();
	printf("delay 20ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	(cbl->delay)(cbl, 200000);
	et = dtime();
	printf("delay 200ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	(cbl->delay)(cbl, 400000);
	et = dtime();
	printf("delay 400ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	(cbl->delay)(cbl, 600000);
	et = dtime();
	printf("delay 600ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);
    }

	jtag_init(cbl);

	st = dtime();
	jtag_IDLE(cbl, 20, 5);
	et = dtime();
	printf("jtag_IDLE 20ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	jtag_IDLE(cbl, 20, 5);
	et = dtime();
	printf("jtag_IDLE 20ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	jtag_IDLE(cbl, 200, 5);
	et = dtime();
	printf("jtag_IDLE 200ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	r = 0;
//verbose = 256;
/*
 * ! Shift in IDCODE(0x16) instruction
 * SIR     8       TDI  (16);
 * SDR     32      TDI  (FFFFFFFF)
 *                 TDO  (0128D043)
 *                 MASK (FFFFFFFF);
 */
	jtag_SIR32(cbl, LSCC_IDCODE, 8, 1, &r);
	printf ("jtag idcode I = %08x\n" ,r);
fflush(stdout);
	jtag_SDR32(cbl, 0xffffffff, 32, 1, &r);
	printf ("jtag idcode D = %08x\n" ,r);

	jtag_IDLE(cbl, 1, 5);
//verbose = 256;
	jtag_SIR32(cbl, LSCC_IDCODE, 8, 1, &r);
	printf ("jtag idcode I = %08x\n" ,r);
fflush(stdout);
	jtag_SDR32(cbl, 0xffffffff, 32, 1, &r);
	printf ("jtag idcode D = %08x\n" ,r);
verbose = 1;
/* 
 * ! Shift in READ USERCODE(0x17) instruction
 * SIR     8       TDI  (17);
 * RUNTEST IDLE    5 TCK   1.00E-003 SEC;
 * SDR     32      TDI  (FFFFFFFF)
 *                 TDO  (FFFFFFFF)
 *                 MASK (00000000);
 */
	jtag_SIR32(cbl, LSCC_USERCODE, 8, 1, &r);
	printf ("jtag usercode I = %08x\n" ,r);
	jtag_IDLE(cbl, 1, 5);
	jtag_SDR32(cbl, 0xffffffff, 32, 1, &r);
	printf ("jtag usercode D = %08x\n" ,r);
	jtag_IDLE(cbl, 1, 5);
}

static void spi_test(TARGET tgt, CABLE cbl)
{
	int i,j;
	uint32_t s,r;
	printf ("spi_test -- sel_chan test\n");
	fflush(stdout);
	for (i=0; i< 4; i++) {
		jtag_reset(cbl);
		(tgt->sel_chan)(tgt, cbl, (i&1)?1:2);
		for (j=0; j< 5; j++) {
			usleep(100000);
		}
	}
	printf ("spi_test -- spi_echo test1\n");
	fflush(stdout);
	jtag_reset(cbl);
	(tgt->sel_chan)(tgt, cbl, 1);
	for (i=0; i< 12; i++) {
		s = (i<10)?(('0'+i) & 0xff):0xff;
		if (i==5) s |= 0x5500;
		if (i==6) s |= 0xaa5500;
//if ((i==5) || (i==6)) verbose = 256;
		jtag_SDR32R(cbl, s , (i==6)?24:(i==5)?16:8, 1, &r);
		printf ("  %02x %02x (%02x)\n" , s, r, r>>1);

verbose = 1;
		fflush(stderr);
		fflush(stdout);
		for (j=0; j< 2; j++) {
			usleep(100000);
		}
	}
	jtag_reset(cbl);
	printf ("spi_test done\n");
	fflush(stdout);
}

extern char *conf_getenv(char *name);

int main(int argc, char *argv[]) {
	char *p,*q;
	int r;
	int c;
	char *port = "ft0";
	char *cbl_name = "mpsse";
	char *tgt_name = "lscc";
	char *device_name = "LCMXO2280";
	int bitclock = 115200;
	int baudrate = 115200;
	int ispdelay = 0;
	char buf[256];

	build_rev();

	program = argv[0];
printf("program : [%s]\n", program);
	p = strrchr(program, '\\');
	q = strrchr(program, '/');
	if (p && !q) progname = p+1;
	else if (!p && q) progname = q+1;
	else if (p && q && p > q) progname = p+1;
	else if (p && q && p <= q) progname = q+1;
	else progname = program;
printf("progname : [%s]\n", progname);
fflush(stdout);
	cbl = malloc(sizeof(struct cable));
	memset(cbl, 0 , sizeof(struct cable));
	tgt = malloc(sizeof(struct target));
	memset(tgt, 0 , sizeof(struct target));

	while ((c=getopt(argc, argv, "c:T:p:P:B:b:i:")) != EOF) {
	    switch (c) {
	    case 'c' :
		cbl_name = optarg;
		break;
	    case 'T' :
		tgt_name = optarg;
		break;
	    case 'p' :
		device_name = optarg;
		break;
	    case 'P' :
		port = optarg;
		break;
	    case 'B' :
		bitclock = atoi(optarg);
		break;
	    case 'b' :
		baudrate = atoi(optarg);
		break;
	    case 'i' :
		ispdelay = atoi(optarg);
		break;
	    }
	}
	strcpy(buf, cbl_name);
	strcat(buf, ".cbl_type");
	p = conf_getenv(buf);
	if (!p) {
		fprintf(stderr, "%s : cable type not defined\n", buf);
		exit(1);
	}
	cbl->cbl_name = cbl_name;
	cbl->bitclock = (double)bitclock;
	cbl->baudrate = baudrate;
	cbl->ispdelay = ispdelay;
	cbl->port = port;
	if (!strcmp(p, "ft245r")) {
		ft245r_init(cbl);
	} else if (!strcmp(p, "serjtag")) {
		serjtag_init(cbl);
	} else {
		fprintf(stderr, "%s : unkown cable type\n", p);
		exit(1);
	}
	if (verbose) {
	    fprintf(stderr, "   cbl_name = %s\n",cbl->cbl_name);
	    fprintf(stderr, "   cbl_type = %s\n",p);
	    fprintf(stderr, "   port     = %s\n",cbl->port);
	    fprintf(stderr, "   bitclock = %8.3f\n",cbl->bitclock);
	    fprintf(stderr, "   baudrate = %8d\n",cbl->baudrate);
	    fprintf(stderr, "   ispdelay = %8d\n",cbl->ispdelay);
	    fflush(stderr);
	}
	r = (cbl->open)(cbl, port);
	if (r) {
		fprintf(stderr, "(cbl->open) error\n");
		exit(1);
	}
printf(" open OK\n");
fflush(stdout);
	(cbl->set_bitclock)(cbl, (double)bitclock);
printf("bitclock = %10.3f \n",cbl->bitclock);
fflush(stdout);
	jtag_test(cbl);

	strcpy(buf, device_name);
	strcat(buf, ".tgt_type");
	p = conf_getenv(buf);
	if (!p) {
	    fprintf(stderr, "target device %s is not defined\n", device_name);
		exit(1);
	}
	r = 0;
	if (!strcmp(p, "lscc")) {
		r = tgtlscc_init(tgt, cbl, device_name);
	} else if (!strcmp(p, "xil")) {
		r = tgtxil_init(tgt, cbl, device_name);
	} else {
		fprintf(stderr, "%s : unkown target type\n", p);
		exit(1);
	}
	if (r) {
	    fprintf(stderr, "target device %s init error\n", device_name);
		exit(1);
	}
	if (verbose) {
//	    fprintf(stderr, "   cbl_name = %s\n",cbl->cbl_name);
	}
fflush(stdout);
fflush(stderr);
	spi_test(tgt, cbl);
fflush(stdout);
fflush(stderr);
	(cbl->close)(cbl, 1);
	return 0;
}
