#ifndef serjtag_h
#define serjtag_h

/* JTAG COMMAND format
 *
 *  +-----------+-----+-----+-----------+-----------+    +-----------+
 *  | COMMONAD  |FLAGS|BITS |   BYTES   |   DATA 1  |... |   DATA N  |
 *  +-----------+-----+-----+-----------+-----------+    +-----------+
 *       8         5     3       8
 *
 *  N : BYTES + (BITS != 0)?1:0 
 *  DATA: MSB first bit-stream
 *  DATA length : 8 * BYTES + BITS
 *
 */

#define JTAG_BUFSIZE	64	// max bytes (max packet size = 3+64)

/* flags */
#define JTAG_RECIEVE	0x80
#define JTAG_TMS_HIGH	0x40
#define JTAG_TDI_HIGH	0x20
#define JTAG_USE_DELAY	0x10
#define JTAG_BITS	0x7

/* 's' command (JTAG_SET) flags */
#define JTAG_SET_TDI	0x80
#define JTAG_SET_TMS	0x40
#define JTAG_SET_TCK	0x20
#define JTAG_SET_DELAY	0x0f

// 'S' : get Software             -> "SERJTAG"
// 'V' : get software version     -> "20" or "21" or ..
//        + exit jtag mode
// 'j' : enter jtaag mode         -> "Y"
#define JTAG_CMD_SET_PORT		's' // flags_bits bytes
#define JTAG_CMD_PUT_TDI_STREAM		'd' // flags_bits bytes data
#define JTAG_CMD_PUT_TMS_TDI_STREAM	'D' // flags_bits bytes data
#define JTAG_CMD_GET_TDO_STREAM		'c' // flags_bits bytes
#define JTAG_CMD_REQUEST_DATA		'r'

#ifdef __cplusplus
extern "C" {
#endif

void serjtag_init (CABLE cbl);

#ifdef __cplusplus
}
#endif

#endif /* serjtag_h */
