#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <unistd.h>

#include "util.h"
#include "cable.h"
#include "serjtag.h"
#include "ft245r.h"
#include "jtag.h"

#include "target.h"
#include "tgtlscc.h"
#include "tgtxil.h"

char *program;
char *progname;
int verbose = 1;

CABLE cbl;
TARGET tgt;

extern char *conf_getenv(char *name);

int main(int argc, char *argv[]) {
	char *p,*q;
	int r;
	int c;
	char *port = "ft0";
	char *cbl_name = "mpsse";
	char *tgt_name = "lscc";
	char *device_name = "LCMXO2280";
	int bitclock = 115200;
	int baudrate = 115200;
	int ispdelay = 0;
	char buf[256];

	program = argv[0];
//printf("program : [%s]\n", program);
	p = strrchr(program, '\\');
	q = strrchr(program, '/');
	if (p && !q) progname = p+1;
	else if (!p && q) progname = q+1;
	else if (p && q && p > q) progname = p+1;
	else if (p && q && p <= q) progname = q+1;
	else progname = program;
printf("progname : [%s]\n", progname);
fflush(stdout);
	cbl = malloc(sizeof(struct cable));
	memset(cbl, 0 , sizeof(struct cable));
	tgt = malloc(sizeof(struct target));
	memset(tgt, 0 , sizeof(struct target));

	while ((c=getopt(argc, argv, "c:T:p:P:B:b:i:")) != EOF) {
	    switch (c) {
	    case 'c' :
		cbl_name = optarg;
		break;
	    case 'T' :
		tgt_name = optarg;
		break;
	    case 'p' :
		device_name = optarg;
		break;
	    case 'P' :
		port = optarg;
		break;
	    case 'B' :
		bitclock = atoi(optarg);
		break;
	    case 'b' :
		baudrate = atoi(optarg);
		break;
	    case 'i' :
		ispdelay = atoi(optarg);
		break;
	    }
	}
	strcpy(buf, cbl_name);
	strcat(buf, ".cbl_type");
	p = conf_getenv(buf);
	if (!p) {
		fprintf(stderr, "%s : cable type not defined\n", buf);
		exit(1);
	}
	cbl->cbl_name = cbl_name;
	cbl->bitclock = (double)bitclock;
	cbl->baudrate = baudrate;
	cbl->ispdelay = ispdelay;
	cbl->port = port;
	if (!strcmp(p, "ft245r")) {
		ft245r_init(cbl);
	} else if (!strcmp(p, "serjtag")) {
		serjtag_init(cbl);
	} else {
		fprintf(stderr, "%s : unkown cable type\n", p);
		exit(1);
	}
	if (verbose) {
	    fprintf(stderr, "   cbl_name = %s\n",cbl->cbl_name);
	    fprintf(stderr, "   cbl_type = %s\n",p);
	    fprintf(stderr, "   port     = %s\n",cbl->port);
	    fprintf(stderr, "   bitclock = %8.3f\n",cbl->bitclock);
	    fprintf(stderr, "   baudrate = %8d\n",cbl->baudrate);
	    fprintf(stderr, "   ispdelay = %8d\n",cbl->ispdelay);
	    fflush(stderr);
	}
	strcpy(buf, device_name);
	strcat(buf, ".tgt_type");
	p = conf_getenv(buf);
	if (!strcmp(p, "lscc")) {
		tgtlscc_init(tgt, cbl, device_name);
	} else if (!strcmp(p, "xil")) {
		tgtxil_init(tgt, cbl, device_name);
	} else {
		fprintf(stderr, "%s : unkown cable type\n", p);
		exit(1);
	}
	if (verbose) {
	    fprintf(stderr, "   cbl_name = %s\n",cbl->cbl_name);
	}
	r = (cbl->open)(cbl, port);
	if (r) {
		fprintf(stderr, "(cbl->open) error\n");
		exit(1);
	}
printf(" open OK\n");
fflush(stdout);
	(cbl->set_bitclock)(cbl, (double)bitclock);
printf("bitclock = %10.3f \n",cbl->bitclock);
fflush(stdout);
	term_mode(tgt, cbl);
	(cbl->close)(cbl, 1);
	return 0;
}
