/*
  XC3S50A.tgt_type       = xil
  XC3S50A.device_id      = 0x02210093
  XC3S50A.boundary_width = 373
  XC3S50A.address_size   = 367
  XC3S50A.data_width     = 1184
  XC3S50A.total_size     = 437312 # addr x width + 2784 (width + 1600)

  XC3S200A.tgt_type       = xil
  XC3S200A.device_id      = 0x02218093
  XC3S200A.boundary_width = 637
  XC3S200A.address_size   = 540
  XC3S200A.data_width     = 2208
  XC3S200A.total_size     = 1196128 # addr x width + 3808 (width + 1600)

  XC6SLX9.tgt_type       = xil
  XC6SLX9.device_id      = 0x04001093
  XC6SLX9.boundary_width = 612
  XC6SLX9.address_size   = 2028 # ?
  XC6SLX9.data_width     = 897 # ?
  XC6SLX9.total_size     = 2724704

  XC3S50.tgt_type       = xil
  XC3S50.device_id      = 0x0140C093
  XC3S50.boundary_width = 383
  XC3S50.address_size   = 368
  XC3S50.data_width     = 1184
  XC3S50.total_size     = 439264 #  addr x width + 3552 (width * 3)

  XC3S250E.tgt_type       = xil
  XC3S250E.device_id      = 0x01C1A093
  XC3S250E.boundary_width = 440
  XC3S250E.address_size   = 577
  XC3S250E.data_width     = 2336
  XC3S250E.total_size     = 1353728 #  addr x width + 5856 (width * 2.5 + 16)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <unistd.h>

#include "util.h"
#include "cable.h"
#include "jtag.h"
#include "target.h"
#include "tgtxil.h"

#define DEV_TYPE_SP3 	0x14
#define DEV_TYPE_SP3E	0x1C
#define DEV_TYPE_SP3A	0x22
#define DEV_TYPE_SP3AN	0x26
#define DEV_TYPE_SP6	0x40
#define DEV_VENDOR_XIL	0x093

#define id_to_vendor(id)	((id) & 0xfff)
#define id_to_family(id)	(((id) >> 24) & 0xfe)

struct supported_device {
	int device_id;
	int address_size;
	int data_width;
	int boundary_width; 
	char *device_name;
};
static struct supported_device cur_device;

#if 0
struct supported_device {
	int device_type;
	int boundary_width;
	char *device_name;
} supported_device[] = {
 { DEV_TYPE_SP3A ,  373 , "XC3S50A"},  // 437312
 { DEV_TYPE_SP3A ,  637 , "XC3S200A"}, // 1196128
 { DEV_TYPE_SP6  ,  612 , "XC6SLX9"},  // 2724704
 { 0  , 0 , "" }
};
#endif

static int sel_channel (TARGET tgt, CABLE cbl, int chan) {
	int ret = -1;
	int r;

	jtag_init(cbl);
	if (chan == 1) {
		jtag_SIR32(cbl, XIL_USER1, 6, 1, &r);
	} else if (chan == 2) {
		jtag_SIR32(cbl, XIL_USER2, 6, 1, &r);
	} else {
		return ret;
	}
	if (((r & IRS_CHK) != 0x01) 
		|| !(r & IRS_DONE)
		||  (r & IRS_INIT) ) {
		return ret;
	}
	return 0;
}

static int program(TARGET tgt, CABLE cbl, uint8_t *data, int len) {
	int ret = -1;
	uint32_t device_id;
	uint32_t r;

	(cbl->setup_gpio)(cbl, CABLE_GPIO_PROG, 1);
	(cbl->setup_gpio)(cbl, CABLE_GPIO_M2, 1);
	(cbl->setup_gpio)(cbl, CABLE_GPIO_M1, 0);
	usleep(2000);
	(cbl->setup_gpio)(cbl, CABLE_GPIO_PROG, 0);
	usleep(5000);
	(cbl->setup_gpio)(cbl, CABLE_GPIO_PROG, 1);
	usleep(2000);
	(cbl->setup_gpio)(cbl, CABLE_GPIO_PROG, -1);

	jtag_SIR32(cbl, XIL_IDCODE, 6, 1, &r);
	if (((r & IRS_CHK) != 0x01) 
		|| !(r & IRS_ISC_ENABLE)) {
		goto err;
	}
	jtag_SDR32(cbl, 0, 32, 1, &device_id);
	if (tgt->device_id != device_id) {
		goto err;
	}
	jtag_SIR32(cbl, JPROGRAM, 6, 1, &r);
	usleep(1000);
	jtag_IDLE(cbl, 1, 5); // 1ms idle
	jtag_SIR32(cbl, CFG_IN, 6, 1, &r);
	jtag_SDR(cbl, data, len * 8, 0, NULL);
	jtag_IDLE(cbl, 1, 5); // 1ms idle
	jtag_SIR32(cbl, JSTART, 6, 1, &r);
	usleep(5000);
	jtag_IDLE(cbl, 5, 25); // 5ms idle
	jtag_SIR32(cbl, XIL_USERCODE, 6, 1, &r);
	if (((r & IRS_CHK) != 0x01) 
		|| !(r & IRS_DONE)
		||  (r & IRS_INIT) ) {
		return ret;
	}
	jtag_SDR32(cbl, 0, 32, 1, &(tgt->user_id));

	// success: reverse M1/M2 for LED
	(cbl->setup_gpio)(cbl, CABLE_GPIO_M1, 1);
	(cbl->setup_gpio)(cbl, CABLE_GPIO_M2, 0);
	return 0;
err:
	return ret;
}

static int verify(TARGET tgt, CABLE cbl, uint8_t *buf, int len) {
	int ret = -1;
	return ret;
}

extern char *conf_getenv(char *name);

static int load_conf(char *dev_name, struct supported_device *dev) {
	char buf[256];
	char *p;

	strcpy(buf, dev_name);
	strcat(buf, ".device_id");
	p = conf_getenv(buf);
	if (!p) return -1;
	if (!strncmp("0x", p, 2)) sscanf(p+2,"%x", &dev->device_id);
	else  sscanf(p,"%d", &dev->device_id);

	strcpy(buf, dev_name);
	strcat(buf, ".boundary_width");
	p = conf_getenv(buf);
	if (!p) return -1;
	if (!strncmp("0x", p, 2)) sscanf(p+2,"%x", &dev->boundary_width);
	else  sscanf(p,"%d", &dev->boundary_width);

	strcpy(buf, dev_name);
	strcat(buf, ".address_size");
	p = conf_getenv(buf);
	if (p) {
	    if (!strncmp("0x", p, 2)) sscanf(p+2,"%x", &dev->address_size);
	    else  sscanf(p,"%d", &dev->address_size);
	}

	strcpy(buf, dev_name);
	strcat(buf, ".data_width");
	p = conf_getenv(buf);
	if (p) {
	    if (!strncmp("0x", p, 2)) sscanf(p+2,"%x", &dev->data_width);
	    else  sscanf(p,"%d", &dev->data_width);
	}

	return 0;
}
int tgtxil_init (TARGET tgt, CABLE cbl, char *dev_name) {
	int ret = -1;
	int r;
	struct supported_device *p;
	int dev_type;
	uint32_t device_id;

	tgt->sel_chan 	= NULL;
	tgt->program  	= NULL;
	tgt->verify  	= NULL;
	tgt->device_id 	= 0;
	tgt->user_id 	= 0;

	if (load_conf(dev_name, &cur_device)) return -1;
	p = &cur_device;
	dev_type = id_to_family(device_id);

	jtag_SIR32(cbl, XIL_IDCODE, 6, 1, &r);
	if (((r & IRS_CHK) != 0x01) 
		|| !(r & IRS_DONE)
		||  (r & IRS_INIT) ) {
		return ret;
	}
	jtag_SDR32(cbl, 0, 32, 1, &(tgt->device_id));
	if (id_to_vendor(tgt->device_id) != DEV_VENDOR_XIL) {
		return ret;
	}
	if (id_to_family(tgt->device_id) != dev_type) {
		return ret;
	}

	tgt->sel_chan 	= sel_channel;
	tgt->program  	= program;
	tgt->verify  	= verify;
	return 0;
}
